module Harold
  module Strategies
    class Update
      def initialize(scope)
        @scope = scope
      end

      def call(operations)
        grouped = operations
                  .select(&method(:scope))
                  .group_by(&method(:scope))

        grouped.each_value do |ops|
          next unless updateable?(ops)

          operations << update_operation(ops.last)
          ops.each { |o| operations.delete(o) }
        end

        operations
      end

      private

      def updateable?(ops)
        types = ops.map(&:type).uniq.sort
        types == %i[add delete]
      end

      def scope(operation)
        operation.payload[@scope]
      end

      def update_operation(operation)
        Operation.new(
          operation.id,
          :update,
          operation.payload,
          operation.attributes
        )
      end
    end
  end
end
